# -*- coding: utf-8 -*-

__author__ = 'ibegtin'


from urllib.parse import urlparse, parse_qs

from pyparsing import Word, nums, alphanums, alphas, oneOf, lineStart, lineEnd, Optional, restOfLine, Literal, ParseException, CaselessLiteral


WEBPAGE_EXTENSIONS = [u'php', u'html', u'htm', u'shtml', u'jsp', u'asp', u'aspx']

EXTENSIONS_PAT = Literal('.').suppress() + oneOf(WEBPAGE_EXTENSIONS).setResultsName('ext')
URLPART = alphanums + '-' + '_'

ENG_MONTHS_SHORT = [u'jan', u'feb', u'mar', u'apr', u'may', u'jun', u'jul', u'aug', u'sep', u'oct', u'nov', u'dec']


# English months map
ensh_mname2mon = dict((m, i + 1) for i, m in enumerate(ENG_MONTHS_SHORT) if m)

BASE_MONTH_EN = oneOf(ENG_MONTHS_SHORT, caseless=True).setParseAction(lambda t: ensh_mname2mon[t[0].lower()])

BASE_TOPIC = Literal('/').suppress() + Word(URLPART).setResultsName('topic')
BASE_SUBTOPIC = Literal('/').suppress() + Word(URLPART).setResultsName('subtopic')
BASE_NAME = Literal('/').suppress() + Word(URLPART).setResultsName('name') + Optional('/').suppress()
BASE_NAME_EXT = Literal('/').suppress() + Word(URLPART).setResultsName('name') + EXTENSIONS_PAT

BASE_ID = Literal('/').suppress() + Word(nums, min=1, max=10).setResultsName("id") + Optional('/').suppress()
BASE_ID_EXT = Literal('/').suppress() + Word(nums, min=1, max=10).setResultsName("id") + EXTENSIONS_PAT
BASE_URL_DATE = Literal('/').suppress() + Word(nums, exact=4).setResultsName('year') + Literal('/').suppress() + Word(
    nums, min=1, max=2).setResultsName('month') + Literal('/').suppress() + Word(nums, min=1, max=2).setResultsName(
    'day')
BASE_MERGED_DATE = Literal('/').suppress() + Word(nums, exact=4).setResultsName('year') + Word(nums, min=1,
                                                                                               max=2).setResultsName(
    'month') + Word(nums, min=1, max=2).setResultsName('day')
BASE_DATE_DASH = Literal('/').suppress() + Word(nums, exact=4).setResultsName('year') + Literal('-').suppress() + Word(
    nums, min=1, max=2).setResultsName('month') + Literal('-').suppress() + Word(nums, min=1, max=2).setResultsName(
    'day')
BASE_DATE_DOT = Literal('/').suppress() + Word(nums, min=1, max=2).setResultsName('day') + Literal(
    '.').suppress() + Word(nums, min=1, max=2).setResultsName('month') + Literal('.').suppress() + Word(nums,
                                                                                                        exact=4).setResultsName(
    'year')
BASE_MERGED_DATE_TEXT = Literal('/').suppress() + Word(nums, min=1, max=2).setResultsName('day') + Literal('/').suppress() + BASE_MONTH_EN + Literal('/').suppress() + Word(nums, exact=4).setResultsName('year')

BASE_MERGED_DATE_ENTEXT = Literal('/').suppress() + Word(nums, min=1, max=2).setResultsName(
    'day') +  Literal('/').suppress() + BASE_MONTH_EN + Literal('/').suppress() + Word(nums, exact=4).setResultsName('year')

BASE_ROOT = Literal('/').suppress()

BASE_URL_PATTERNS = {
#    '/' : BASE_ROOT,
    '/[name]/': BASE_NAME,
    '/[name].[ext]': BASE_NAME_EXT,
    '/[topic]/[id]/': BASE_TOPIC + BASE_ID,
    '/[topic]/[id].[ext]': BASE_TOPIC + BASE_ID_EXT,
    '/[topic]/[name].[ext]': BASE_TOPIC + BASE_NAME_EXT,
    '/[topic]/[subtopic]/': BASE_TOPIC + BASE_SUBTOPIC,
    '/[topic]/[subtopic]/[name].[ext]': BASE_TOPIC + BASE_SUBTOPIC + BASE_NAME_EXT,
    '/[topic]/[subtopic]/[id]/': BASE_TOPIC + BASE_SUBTOPIC + BASE_ID,
    '/[topic]/[subtopic]/[id].[ext]': BASE_TOPIC + BASE_SUBTOPIC + BASE_ID_EXT,
    '/[yyyy]/[mm]/[dd]/[id]/': BASE_URL_DATE + BASE_ID,
    '/[yyyy]/[mm]/[dd]/[id].[ext]': BASE_URL_DATE + BASE_ID_EXT,
    '/[topic]/[yyyy]/[mm]/[dd]/': BASE_TOPIC + BASE_URL_DATE,
    '/[topic]/[yyyy]/[mm]/[dd]/[id]/': BASE_TOPIC + BASE_URL_DATE + BASE_ID,
    '/[topic]/[yyyy]/[mm]/[dd]/[id].[ext]': BASE_TOPIC + BASE_URL_DATE + BASE_ID_EXT,
    '/[topic]/[yyyy]/[mtext]/[dd]/[id]/': BASE_TOPIC + BASE_MERGED_DATE_TEXT + BASE_ID,
    '/[topic]/[yyyy]/[mtext]/[dd]/[id].[ext]': BASE_TOPIC + BASE_MERGED_DATE_TEXT + BASE_ID_EXT,
    '/[topic]/[yyyy]/[mtext]/[dd]/[name]/': BASE_TOPIC + BASE_MERGED_DATE_TEXT + BASE_NAME,
    '/[topic]/[yyyy]/[mtext]/[dd]/[name].[ext]': BASE_TOPIC + BASE_MERGED_DATE_TEXT + BASE_NAME_EXT,
    '/[yyyy]/[mm]/[dd]/[name]/': BASE_URL_DATE + BASE_NAME,
    '/[yyyy]/[mm]/[dd]/[name].[ext]': BASE_URL_DATE + BASE_NAME_EXT,
    '/[topic]/[yyyy]-[mm]-[dd]/[id]/': BASE_TOPIC + BASE_DATE_DASH + BASE_ID,
    '/[topic]/[yyyy]-[mm]-[dd]/[id].[ext]/': BASE_TOPIC + BASE_DATE_DASH + BASE_ID_EXT,
    '/[topic]/[dd].[mm].[yyyy]/[id]/': BASE_TOPIC + BASE_DATE_DOT + BASE_ID,
    '/[topic]/[dd].[mm].[yyyy]/[id].[ext]': BASE_TOPIC + BASE_DATE_DOT + BASE_ID_EXT,
    '/[topic]/[subtopic]/[dd].[mm].[yyyy]/[id]/': BASE_TOPIC + BASE_SUBTOPIC + BASE_DATE_DOT + BASE_ID,
    '/[topic]/[subtopic]/[dd].[mm].[yyyy]/[id].[ext]': BASE_TOPIC + BASE_SUBTOPIC + BASE_DATE_DOT + BASE_ID_EXT,
    '/[topic]/[dd][mtext][yyyy]/[name]/': BASE_TOPIC + BASE_MERGED_DATE_ENTEXT + BASE_NAME,
    '/[topic]/[dd][mtext][yyyy]/[name].[ext]': BASE_TOPIC + BASE_MERGED_DATE_ENTEXT + BASE_NAME_EXT,
    '/[topic]/[yyyy][mm][dd]/[id]/': BASE_TOPIC + BASE_MERGED_DATE + BASE_ID,
    '/[topic]/[yyyy][mm][dd]/[id].[ext]': BASE_TOPIC + BASE_MERGED_DATE + BASE_ID_EXT,
    '/[topic]/[yyyy][mm][dd]/[name]/': BASE_TOPIC + BASE_MERGED_DATE + BASE_NAME,
    '/[topic]/[yyyy][mm][dd]/[name].[ext]': BASE_TOPIC + BASE_MERGED_DATE + BASE_NAME_EXT,
    '/[yyyy][mm][dd]/[name]/': BASE_MERGED_DATE + BASE_NAME,
    '/[yyyy][mm][dd]/[name].[ext]': BASE_MERGED_DATE + BASE_NAME_EXT,
}


def improve_base_patterns(patterns):
    result = {}
    for k, v in patterns.items():
        result[k] = lineStart + v + lineEnd
    return result


CLEANABLE_QUERY_KEYS = ['phpsessid', 'utm_source', 'utm_campaign', 'utm_medium', 'utm_content', 'utm_hp_ref']
CLEANABLE_QUERY_PAT = oneOf(CLEANABLE_QUERY_KEYS, caseless=True)

JSESSION_PAT = Word(URLPART).setResultsName('base') + ";jsessionId=" + Word(URLPART + '.').setResultsName('key')

IDKEYS = ['id', 'itemid']
IDKEKEYS_PAT = oneOf(IDKEYS, caseless=True)
IDVALUE_PAT = Word(nums, min=1, max=8).setResultsName('idvalue')

QUERY_URL_PATTERNS = {
    '/[topic]/?[idkey]=[idvalue]': {'path': BASE_NAME, 'qkeypatt': IDVALUE_PAT, 'qvaluepatt': IDVALUE_PAT},
    '/[topic]/[subtopic]/?[idkey]=[idvalue]': {'path': BASE_TOPIC + BASE_NAME, 'qkeypatt': IDVALUE_PAT,
                                               'qvaluepatt': IDVALUE_PAT},
}


FINAL_URL_PATTERNS = improve_base_patterns(BASE_URL_PATTERNS)
#FINAL_URL_PATTERNS = BASE_URL_PATTERNS

NEWS_URLPATS = ['news', 'announcement', 'announcements', 'press-release', 'articles']
# Spanish url parts singular and plural "news"
NEWS_URLPATS.extend(['noticia', 'noticias'])

BASE_NEWS_PATTERNS = [
    {'key' : 'url:wordpress_post', 'name' : 'Wordpress or news site url as /[yyyy]/[mm]/[dd]/[name]/',
             'type' : 'dt:root:string', 'length' : {'min' : 10, 'max' : 150},
             'pat' : BASE_URL_PATTERNS["/[yyyy]/[mm]/[dd]/[name]/"]
    },
    {'key' : 'url:news_id', 'name' : 'News id url like /news/[id]/',
             'type' : 'dt:root:string', 'length' : {'min' : 8, 'max' : 15},
             'pat' : Literal('/').suppress() + oneOf(NEWS_URLPATS, caseless=True).suppress() + BASE_ID
    },
    {'key' : 'url:news_subtopic_id', 'name' : 'News subtopic and id url like /[topic]/[subtopic]/8012',
             'type' : 'dt:root:string', 'length' : {'min' : 8, 'max' : 15},
             'pat' : Literal('/').suppress() + oneOf(NEWS_URLPATS, caseless=True).suppress() + Literal('/').suppress() + oneOf(NEWS_URLPATS, caseless=True).suppress() +  BASE_ID
    },
    {'key' : 'url:bitrix_news_1', 'name' : 'Bitrix style news, format /news/detail/[id]',
             'type' : 'dt:root:string', 'length' : {'min' : 12, 'max' : 150},
             'pat' : Literal('/').suppress() + CaselessLiteral('news').suppress() + Literal('/').suppress() + CaselessLiteral('detail').suppress() + BASE_ID
    },
    {'key' : 'url:bitrix_news_1', 'name' : 'Sitesoft cms style news, format /news/item/[id]',
             'type' : 'dt:root:string', 'length' : {'min' : 10, 'max' : 150},
             'pat' : Literal('/').suppress() + CaselessLiteral('news').suppress() + Literal('/').suppress() + CaselessLiteral('item').suppress() + BASE_ID
    },
    {'key' : 'url:dated_news_topic', 'name' : 'News for dated topic with id like /[topic]/[yyyy]/[mm]/[dd]/[id]/',
             'type' : 'dt:root:string', 'length' : {'min' : 13, 'max' : 150},
             'pat' : BASE_URL_PATTERNS['/[topic]/[yyyy]/[mm]/[dd]/[id]/']
    },
    {'key' : 'url:dated_id', 'name' : 'Url with date and with id like /[yyyy]/[mm]/[dd]/[id]/',
             'type' : 'dt:root:string', 'length' : {'min' : 11, 'max' : 150},
             'pat' : BASE_URL_PATTERNS['/[yyyy]/[mm]/[dd]/[id]/']
    },
]

BASE_QUERY_NEWS_PATTERNS = [
    {'key' : 'url:query_news_id', 'name' : 'Query news id url like /news/?id=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 10, 'max' : 17},
             'pat' : Literal('/').suppress() + CaselessLiteral('news').suppress() + Literal('?').suppress() + CaselessLiteral('id=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
    {'key' : 'url:query_news_id', 'name' : 'Bitrix query news id url like /news.php?ELEMENT_ID=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 20, 'max' : 28},
             'pat' : Literal('/').suppress() + CaselessLiteral('news.php').suppress() + Literal('?').suppress() + Literal('ELEMENT_ID=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
    {'key' : 'url:query_news_id', 'name' : 'Bitrix query news id url like /news/?ELEMENT_ID=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 16, 'max' : 25},
             'pat' : Literal('/').suppress() + CaselessLiteral('news').suppress() + Optional('/').suppress() + Literal('?').suppress() + Literal('ELEMENT_ID=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
    {'key' : 'url:query_news_index_id', 'name' : 'Bitrix query news with id, index page and url like /news/index.php?ELEMENT_ID=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 16, 'max' : 30},
             'pat' : Literal('/').suppress() + CaselessLiteral('news/index.php').suppress() + Literal('?').suppress() + Literal('ELEMENT_ID=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
    {'key' : 'url:query_topic_news_id', 'name' : 'Bitrix url with news id, topic and query  like /about/news/?ELEMENT_ID=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 16, 'max' : 150},
             'pat' : BASE_TOPIC + Literal('/').suppress() + CaselessLiteral('news').suppress() + Optional('/').suppress() + Literal('?').suppress() + Literal('ELEMENT_ID=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
    {'key' : 'url:query_subtopic_news_id', 'name' : 'Bitrix url with news id, topic, subtopic and query  like /about/press/news/?ELEMENT_ID=8012',
             'type' : 'dt:root:string', 'length' : {'min' : 16, 'max' : 150},
             'pat' : BASE_TOPIC + BASE_SUBTOPIC + Literal('/').suppress() + oneOf(NEWS_URLPATS, caseless=True).suppress() + Optional('/').suppress() + Literal('?').suppress() + Literal('ELEMENT_ID=').suppress() + Word(nums, min=1, max=10).setResultsName("id")
    },
]

# Used to march parts of url string
NEWS_URLPAT_KEYS = NEWS_URLPATS



NEWS_HEADERS_TEXTS_RUS = [u'Новости', u'НОВОСТИ', u'новости']
NEWS_HEADERS_TEXTS_EN = [u'news',]
NEWS_HEADERS_TEXTS_ES = [u'noticias', ]

ALL_NEWS_HEADERS_TEXTS = NEWS_HEADERS_TEXTS_RUS
ALL_NEWS_HEADERS_TEXTS.extend(NEWS_HEADERS_TEXTS_EN)
ALL_NEWS_HEADERS_TEXTS.extend(NEWS_HEADERS_TEXTS_ES)

def improve_news_patterns(patterns):
    """Adds line start and line end to pattern to avoid partial url matching"""
    results = []
    for p in patterns:
        p['pat'] = p['pat'] = lineStart + p['pat'] + lineEnd
        results.append(p)
    return results

ALL_NEWS_PATTERNS = improve_news_patterns(BASE_NEWS_PATTERNS)
ALL_NEWS_PATTERNS.extend(improve_news_patterns(BASE_QUERY_NEWS_PATTERNS))


def clean_url(url):
    """Removes just query parameters from url"""

    # clean url from jsession param
    THE_JS_KEY  = ';jsessionid='
    n = url.find(THE_JS_KEY)
    if n > -1:
        thepath = url[:n] + url[n+len(THE_JS_KEY) + 32:]
        url = thepath
    o = urlparse(url)

    # clean query
    if len(o.query) > 0:
        query = clean_urlquery(o.query)[0]
        if len(query) > 0:
            url = o.geturl().rsplit('?')[0] + '?' + query
        else:
            return o.geturl().rsplit('?')[0]
        return url
#    print o
    return o.geturl()

def clean_urlquery(qs):
    """Removes _junk_ query parameters left by analytics systems"""
    items = parse_qs(qs, keep_blank_values=True)
    results = {}
    filtered = {}
    for k, v in items.items():
        if k.lower() not in CLEANABLE_QUERY_KEYS:
            results[k] = v
        else:
            filtered[k] = v
    q = []
    for k, v in results.items():
        q.append('%s=%s' % (k, v[0]))
    query = '&'.join(q)
    return query, results, filtered



tn = {}
